@@ -67,8 +67,9 @@
 #'   below.
 #' @param extract_model_data A function for fetching some variables (response,
 #'   observation weights, offsets) from the original dataset (supplied to
-#'   argument `data`) or from a new dataset. See also section "Argument
-#'   `extract_model_data`" below.
+#'   argument `data`) or from a new dataset. May be `NULL` for using an internal
+#'   default that essentially corresponds to [y_wobs_offs()]. See also section
+#'   "Argument `extract_model_data`" below.
 #' @param family An object of class `family` representing the observation model
 #'   (i.e., the distributional family for the response) of the *submodels*.
 #'   (However, the link and the inverse-link function of this `family` are also
@@ -90,11 +91,10 @@
 #'   `cvfits` is used.
 #' @param cvfun For \eqn{K}-fold CV only. A function that, given a fold indices
 #'   vector, fits the reference model separately for each fold and returns the
-#'   \eqn{K} model fits as a `list`. Each of the \eqn{K} model fits needs to be
-#'   a `list`. If `object` is `NULL`, `cvfun` may be `NULL` for using an
-#'   internal default. Only one of `cvfits` and `cvfun` needs to be provided
-#'   (for \eqn{K}-fold CV). Note that `cvfits` takes precedence over `cvfun`,
-#'   i.e., if both are provided, `cvfits` is used.
+#'   \eqn{K} model fits as a `list`. If `object` is `NULL`, `cvfun` may be
+#'   `NULL` for using an internal default. Only one of `cvfits` and `cvfun`
+#'   needs to be provided (for \eqn{K}-fold CV). Note that `cvfits` takes
+#'   precedence over `cvfun`, i.e., if both are provided, `cvfits` is used.
 #' @param cvrefbuilder For \eqn{K}-fold CV only. A function that, given a
 #'   reference model fit for fold \eqn{k \in \{1, ..., K\}}{k = 1, ..., K},
 #'   returns an object of the same type as [init_refmodel()] does. The reference
@@ -171,7 +171,7 @@
 #' following prototypes:
 #' * `ref_predfun`: `ref_predfun(fit, newdata = NULL)` where:
 #'     + `fit` accepts the reference model fit as given in argument `object`
-#'     (but possibly re-fitted to a subset of the observations, as done in
+#'     (but possibly refitted to a subset of the observations, as done in
 #'     \eqn{K}-fold CV).
 #'     + `newdata` accepts either `NULL` (for using the original dataset,
 #'     typically stored in `fit`) or data for new observations (at least in the
@@ -205,14 +205,13 @@
 #'     GLM fitter) in case of the traditional or the latent projection and an
 #'     \eqn{N_{\mathrm{augcat}} \times S_{\mathrm{prj}}}{N_augcat x S_prj}
 #'     matrix (containing only `NA`s) in case of the augmented-data projection.
-#'     + `projpred_regul` accepts a single numeric value as supplied to argument
-#'     `regul` of [project()], for example.
 #'     + `projpred_ws_aug` accepts an \eqn{N \times S_{\mathrm{prj}}}{N x S_prj}
 #'     matrix of expected values for the response in case of the traditional or
 #'     the latent projection and an \eqn{N_{\mathrm{augcat}} \times
 #'     S_{\mathrm{prj}}}{N_augcat x S_prj} matrix of probabilities for the
 #'     response categories in case of the augmented-data projection.
-#'     + `...` accepts further arguments specified by the user.
+#'     + `...` accepts further arguments specified by the user (or by
+#'     \pkg{projpred}).
 #'
 #' The return value of these functions needs to be:
 #' * `ref_predfun`: for the traditional or the latent projection, an \eqn{N
@@ -239,20 +238,20 @@
 #' ```
 #' where:
 #' * `object` accepts the reference model fit as given in argument `object` (but
-#' possibly re-fitted to a subset of the observations, as done in \eqn{K}-fold
+#' possibly refitted to a subset of the observations, as done in \eqn{K}-fold
 #' CV).
 #' * `newdata` accepts data for new observations (at least in the form of a
 #' `data.frame`).
 #' * `wrhs` accepts at least (i) a right-hand side formula consisting only of
 #' the variable in `newdata` containing the observation weights or (ii) `NULL`
-#' (for typical \pkg{rstanarm} and \pkg{brms} reference models, `NULL` causes
-#' the original observation weights to be used if the model was fitted with
-#' weights, otherwise a vector of ones is used).
+#' for using the observation weights corresponding to `newdata` (typically, the
+#' observation weights are stored in a column of `newdata`; if the model was
+#' fitted without observation weights, a vector of ones should be used).
 #' * `orhs` accepts at least (i) a right-hand side formula consisting only of
-#' the variable in `newdata` containing the offsets or (ii) `NULL` (for typical
-#' \pkg{rstanarm} and \pkg{brms} reference models, `NULL` causes the original
-#' offsets to be used if the model was fitted with offsets, otherwise a vector
-#' of zeros is used).
+#' the variable in `newdata` containing the offsets or (ii) `NULL` for using the
+#' offsets corresponding to `newdata` (typically, the offsets are stored in a
+#' column of `newdata`; if the model was fitted without offsets, a vector of
+#' zeros should be used).
 #' * `extract_y` accepts a single logical value indicating whether output
 #' element `y` (see below) shall be `NULL` (`TRUE`) or not (`FALSE`).
 #'
@@ -265,6 +264,12 @@
 #' The weights and offsets returned by `extract_model_data` will be assumed to
 #' hold for the reference model as well as for the submodels.
 #'
+#' Above, arguments `wrhs` and `orhs` were assumed to have defaults of `NULL`.
+#' It should be possible to use defaults other than `NULL`, but we strongly
+#' recommend to use `NULL`. If defaults other than `NULL` are used, they need to
+#' imply the behaviors described at items "(ii)" (see the descriptions of `wrhs`
+#' and `orhs`).
+#'
 #' # Augmented-data projection
 #'
 #' If a custom reference model for an augmented-data projection is needed, see
@@ -317,7 +322,7 @@
 #' # Data:
 #' dat_gauss <- data.frame(y = df_gaussian$y, df_gaussian$x)
 #'
-#' # The "stanreg" fit which will be used as the reference model (with small
+#' # The `stanreg` fit which will be used as the reference model (with small
 #' # values for `chains` and `iter`, but only for technical reasons in this
 #' # example; this is not recommended in general):
 #' fit <- rstanarm::stan_glm(
@@ -335,20 +340,12 @@
 #'
 #' # A custom reference model object which may be used in a variable selection
 #' # where the candidate predictors are not a subset of those used for the
-#' # reference model's predictions (defining the function for argument
-#' # `extract_model_data` first because it can be re-used for the `cvrefbuilder`
-#' # function here):
-#' extractor_cust <- function(object, newdata, wrhs = NULL, orhs = NULL,
-#'                            extract_y = TRUE) {
-#'   return(y_wobs_offs(newdata = newdata, wrhs = wrhs, orhs = orhs,
-#'                      resp_form = if (extract_y) ~ y else NULL))
-#' }
+#' # reference model's predictions:
 #' ref_cust <- init_refmodel(
 #'   fit,
 #'   data = dat_gauss,
 #'   formula = y ~ X6 + X7,
 #'   family = gaussian(),
-#'   extract_model_data = extractor_cust,
 #'   cvfun = function(folds) {
 #'     kfold(
 #'       fit, K = max(folds), save_fits = TRUE, folds = folds, cores = 1
@@ -360,7 +357,6 @@
 #'                   data = dat_gauss[-cvfit$omitted, , drop = FALSE],
 #'                   formula = y ~ X6 + X7,
 #'                   family = gaussian(),
-#'                   extract_model_data = extractor_cust,
 #'                   dis = as.matrix(cvfit)[, "sigma"],
 #'                   called_from_cvrefbuilder = TRUE)
 #'   }
@@ -372,6 +368,53 @@ NULL
 
 # Function definitions ----------------------------------------------------
 
+#' Print information about a reference model object
+#'
+#' This is the [print()] method for reference model objects (objects of class
+#' `refmodel`). This method mainly exists to avoid cluttering the console when
+#' printing such objects accidentally.
+#'
+#' @param x An object of class `refmodel` (returned by [get_refmodel()] or
+#'   [init_refmodel()]).
+#' @param ... Currently ignored.
+#'
+#' @return The input object `x` (invisible).
+#'
+#' @export
+print.refmodel <- function(x, ...) {
+  cat_cls(x)
+  # Print information about `x` (the order of that information is from most
+  # strongly tied uniquely to the reference model (top) to most strongly tied
+  # uniquely to the submodels (bottom)).
+  cat("Class of `fit` (first class only): ", utils::head(class(x$fit), 1), "\n",
+      sep = "")
+  if (!inherits(x, "datafit")) {
+    cat("Number of posterior draws: ", length(x$wdraws_ref), "\n", sep = "")
+  }
+  cat("Number of observations: ", x$nobs, "\n", sep = "")
+  if (x$family$for_augdat) {
+    prj_meth <- "augmented-data"
+  } else if (x$family$for_latent) {
+    prj_meth <- "latent"
+  } else {
+    prj_meth <- "traditional"
+  }
+  cat("Projection method: ", prj_meth, "\n", sep = "")
+  if (x$family$for_latent) {
+    cat("------\nResponse-scale family:\n")
+    print(structure(x$family[c("family_oscale", "link_oscale")],
+                    class = "family"))
+    cat("------\nLatent-scale family:\n")
+  }
+  print(x$family)
+  if (x$family$for_latent) {
+    cat("------\n")
+  }
+  cat("Formula: ")
+  print(x$formula, showEnv = FALSE)
+  return(invisible(x))
+}
+
 #' Predictions or log posterior predictive densities from a reference model
 #'
 #' This is the [predict()] method for `refmodel` objects (returned by
@@ -438,7 +481,7 @@ predict.refmodel <- function(object, newdata = NULL, ynew = NULL,
                              offsetnew = NULL, weightsnew = NULL,
                              type = "response", ...) {
   if (inherits(object, "datafit")) {
-    stop("Cannot make predictions for an `object` of class \"datafit\".")
+    stop("Cannot make predictions for an `object` of class `datafit`.")
   }
   refmodel <- object
   if (!type %in% c("response", "link")) {
@@ -476,28 +519,11 @@ predict.refmodel <- function(object, newdata = NULL, ynew = NULL,
   if (!is.null(newdata)) {
     newdata <- na.fail(newdata)
   }
-  nobs_new <- nrow(newdata) %||% refmodel$nobs
   w_o <- refmodel$extract_model_data(refmodel$fit, newdata = newdata,
                                      wrhs = weightsnew, orhs = offsetnew,
                                      extract_y = FALSE)
   weightsnew <- w_o$weights
   offsetnew <- w_o$offset
-  if (length(weightsnew) == 0) {
-    stop("The function supplied to argument `extract_model_data` of ",
-         "init_refmodel() must not return a length-zero element `weights`.")
-  }
-  if (length(offsetnew) == 0) {
-    stop("The function supplied to argument `extract_model_data` of ",
-         "init_refmodel() must not return a length-zero element `offset`.")
-  }
-  if (refmodel$family$for_augdat && !all(weightsnew == 1)) {
-    stop("Currently, the augmented-data projection may not be combined with ",
-         "observation weights (other than 1).")
-  }
-  if (refmodel$family$for_latent && !all(weightsnew == 1)) {
-    stop("Currently, the latent projection may not be combined with ",
-         "observation weights (other than 1).")
-  }
   if (!is.null(newdata) && inherits(refmodel$fit, "stanreg") &&
       length(refmodel$fit$offset) > 0) {
     if ("projpred_internal_offs_stanreg" %in% names(newdata)) {
@@ -540,18 +566,12 @@ predict.refmodel <- function(object, newdata = NULL, ynew = NULL,
     was_augmat <- inherits(pred, "augmat")
     ## integrate over the draws
     if (type == "link" || !refmodel$family$for_latent || was_augmat) {
-      if (ncol(pred) > 1) {
-        pred <- rowMeans(pred)
-      }
+      pred <- rowMeans(pred)
     } else {
-      if (nrow(pred) > 1) {
-        pred <- colMeans(pred)
-      }
+      pred <- colMeans(pred)
     }
     if (was_augmat) {
-      pred <- structure(pred, nobs_orig = nobs_new, class = "augvec")
-      pred <- augmat2arr(augvec2augmat(pred))
-      pred <- matrix(pred, nrow = dim(pred)[1], ncol = dim(pred)[2])
+      pred <- matrix(pred, nrow = nrow(newdata) %||% refmodel$nobs)
     }
     return(pred)
   } else {
@@ -565,7 +585,7 @@ predict.refmodel <- function(object, newdata = NULL, ynew = NULL,
         stop("Unexpected structure for the output of `latent_ilink`.")
       }
       loglik <- refmodel$family$latent_ll_oscale(
-        mu_oscale, y_oscale = ynew, wobs = weightsnew,
+        mu_oscale, dis = refmodel$dis, y_oscale = ynew, wobs = weightsnew,
         cl_ref = seq_along(refmodel$wdraws_ref),
         wdraws_ref = rep(1, length(refmodel$wdraws_ref))
       )
@@ -590,8 +610,8 @@ predict.refmodel <- function(object, newdata = NULL, ynew = NULL,
         if (all(is.na(refmodel$dis))) {
           message(
             "Cannot calculate LPD values if `type = \"link\"` and ",
-            "`<refmodel>$dis` consists of only `NA`s. If it's not possible to ",
-            "supply a suitable argument `dis` to init_refmodel(), consider ",
+            "`<refmodel>$dis` consists of only `NA`s. If it is not possible to ",
+            "supply values to argument `dis` of init_refmodel(), consider ",
             "switching to `type = \"response\"` (which might require the ",
             "specification of functions needed by extend_family())."
           )
@@ -688,9 +708,6 @@ refprd <- function(fit, newdata = NULL) {
 #'
 #' @seealso [init_refmodel()]
 #'
-#' @examples
-#' # For an example, see `?init_refmodel`.
-#'
 #' @export
 y_wobs_offs <- function(newdata, wrhs = NULL, orhs = NULL, resp_form) {
   if (inherits(wrhs, "formula")) {
@@ -742,32 +759,29 @@ get_refmodel <- function(object, ...) {
 #' @rdname refmodel-init-get
 #' @export
 get_refmodel.refmodel <- function(object, ...) {
-  # If the object is already of class "refmodel", then simply return it as is:
+  # If the object is already of class `refmodel`, then simply return it as is:
   object
 }
 
 #' @rdname refmodel-init-get
 #' @export
 get_refmodel.vsel <- function(object, ...) {
-  # The reference model is stored in the `object` of class "vsel":
+  # The reference model is stored in the `object` of class `vsel`:
   object$refmodel
 }
 
 #' @rdname refmodel-init-get
 #' @export
-get_refmodel.default <- function(object, data, formula, family = NULL, ...) {
-  return(init_refmodel(
-    object = object,
-    data = data,
-    formula = formula,
-    family = family %||% family(object),
-    extract_model_data = function(object, newdata, wrhs = NULL, orhs = NULL,
-                                  extract_y = TRUE) {
-      return(y_wobs_offs(newdata = newdata, wrhs = wrhs, orhs = orhs,
-                         resp_form = if (extract_y) lhs(formula) else NULL))
-    },
-    ...
-  ))
+get_refmodel.projection <- function(object, ...) {
+  # The reference model is stored in the `object` of class `projection`:
+  object$refmodel
+}
+
+#' @rdname refmodel-init-get
+#' @export
+get_refmodel.default <- function(object, family = NULL, ...) {
+  return(init_refmodel(object = object, family = family %||% family(object),
+                       ...))
 }
 
 #' @rdname refmodel-init-get
@@ -1049,7 +1063,7 @@ get_refmodel.stanreg <- function(object, latent = FALSE, dis = NULL, ...) {
     ref_predfun = ref_predfun, extract_model_data = extract_model_data,
     dis = dis, cvfun = cvfun, cvrefbuilder = cvrefbuilder
   )
-  return(do.call(init_refmodel, args = c(args_basic, args_augdat, args_latent,
+  return(do_call(init_refmodel, args = c(args_basic, args_augdat, args_latent,
                                          list(...))))
 }
 
@@ -1057,7 +1071,7 @@ get_refmodel.stanreg <- function(object, latent = FALSE, dis = NULL, ...) {
 #' @export
 init_refmodel <- function(object, data, formula, family, ref_predfun = NULL,
                           div_minimizer = NULL, proj_predfun = NULL,
-                          extract_model_data, cvfun = NULL,
+                          extract_model_data = NULL, cvfun = NULL,
                           cvfits = NULL, dis = NULL, cvrefbuilder = NULL,
                           called_from_cvrefbuilder = FALSE, ...) {
   # Family ------------------------------------------------------------------
@@ -1072,26 +1086,6 @@ init_refmodel <- function(object, data, formula, family, ref_predfun = NULL,
     }
   }
 
-  family$mu_fun <- function(fits, obs = NULL, newdata = NULL, offset = NULL,
-                            transform = TRUE) {
-    newdata <- fetch_data(data, obs = obs, newdata = newdata)
-    if (is.null(offset)) {
-      offset <- rep(0, nrow(newdata))
-    } else {
-      stopifnot(length(offset) %in% c(1L, nrow(newdata)))
-    }
-    pred_sub <- proj_predfun(fits, newdata = newdata)
-    if (family$family %in% fams_neg_linpred()) {
-      pred_sub <- pred_sub - offset
-    } else {
-      pred_sub <- pred_sub + offset
-    }
-    if (transform) {
-      pred_sub <- family$linkinv(pred_sub)
-    }
-    return(pred_sub)
-  }
-
   if (family$family == "categorical" && family$link != "logit") {
     stop("For the brms::categorical() family, projpred only supports the ",
          "logit link.")
@@ -1182,11 +1176,48 @@ init_refmodel <- function(object, data, formula, family, ref_predfun = NULL,
 
   # Functions ---------------------------------------------------------------
 
-  # Wrap `extract_model_data` in order to retrieve the correct `newdata` when
-  # `newdata` is `NULL`:
   extract_model_data_usr <- extract_model_data
+  if (is.null(extract_model_data_usr)) {
+    # The internal default for `extract_model_data`:
+    extract_model_data_usr <- function(object, newdata, wrhs = NULL,
+                                       orhs = NULL, extract_y = TRUE) {
+      if (extract_y) {
+        resp_fml <- lhs(formula)
+        if (family$for_latent) {
+          resp_fml <- update(resp_fml,
+                             paste(sub("^\\.", "", as.character(resp_fml[[2]])),
+                                   "~ ."))
+        }
+      } else {
+        resp_fml <- NULL
+      }
+      return(y_wobs_offs(newdata = newdata, wrhs = wrhs, orhs = orhs,
+                         resp_form = resp_fml))
+    }
+  }
+  # Wrap `extract_model_data_usr`:
   extract_model_data <- function(object, newdata, ...) {
-    extract_model_data_usr(object = object, newdata = newdata %||% data, ...)
+    newdata <- newdata %||% data
+    mdat <- extract_model_data_usr(object = object, newdata = newdata, ...)
+    if (length(mdat$weights) != nrow(newdata)) {
+      stop("The function supplied to argument `extract_model_data` of ",
+           "init_refmodel() needs to return an element `weights` with length ",
+           "equal to the number of observations.")
+    }
+    if (length(mdat$offset) != nrow(newdata)) {
+      stop("The function supplied to argument `extract_model_data` of ",
+           "init_refmodel() needs to return an element `offset` with length ",
+           "equal to the number of observations.")
+    }
+    if (family$for_augdat && !all(mdat$weights == 1)) {
+      stop("Currently, the augmented-data projection may not be combined with ",
+           "observation weights (other than 1).")
+    }
+    if (family$for_latent && !all(mdat$weights == 1)) {
+      stop("Currently, the latent projection may not be combined with ",
+           "observation weights (other than 1).")
+    }
+    return(mdat)
   }
 
   if (proper_model) {
@@ -1302,8 +1333,8 @@ init_refmodel <- function(object, data, formula, family, ref_predfun = NULL,
         # `?init_refmodel` for a definition of these dimensions). Therefore, it
         # is converted to an augmented-rows matrix (see `?`augdat-internals``
         # for a definition):
+        n_obs <- dim(linpred_out)[2]
         linpred_out <- arr2augmat(linpred_out, margin_draws = 1)
-        n_obs <- attr(linpred_out, "nobs_orig")
       } else {
         stop("Unexpected structure for `linpred_out`. Does the return value ",
              "of `ref_predfun` have the correct structure?")
@@ -1311,10 +1342,10 @@ init_refmodel <- function(object, data, formula, family, ref_predfun = NULL,
       linpred_out <- unname(linpred_out)
 
       if (excl_offs) {
-        # Observation weights are not needed here, so use a vector of ones for
-        # `wrhs` to avoid potential conflicts for a non-`NULL` default `wrhs`:
+        # Observation weights are not needed here, so we can use the default of
+        # `wrhs = NULL` (using something like `wrhs = rep(1, nrow(newdata %||%
+        # data))` would cause a warning in newer brms versions):
         offs <- extract_model_data(fit, newdata = newdata,
-                                   wrhs = rep(1, nrow(newdata %||% data)),
                                    extract_y = FALSE)$offset
         stopifnot(length(offs) %in% c(1L, n_obs))
         if (family$family %in% fams_neg_linpred()) {
@@ -1333,11 +1364,7 @@ init_refmodel <- function(object, data, formula, family, ref_predfun = NULL,
                             mlvl_allrandom = getOption("projpred.mlvl_pred_new",
                                                        FALSE)) {
       stopifnot(is.null(fit))
-      if (is.null(newdata)) {
-        return(matrix(rep(NA_real_, nrow(data))))
-      } else {
-        return(matrix(rep(NA_real_, nrow(newdata))))
-      }
+      return(matrix(rep(NA_real_, nrow(newdata %||% data))))
     }
   }
 
@@ -1366,6 +1393,26 @@ init_refmodel <- function(object, data, formula, family, ref_predfun = NULL,
     }
   }
 
+  mu_fun <- function(fits, obs = NULL, newdata = NULL, offset = NULL,
+                     transform = TRUE) {
+    newdata <- fetch_data(data, obs = obs, newdata = newdata)
+    if (is.null(offset)) {
+      offset <- rep(0, nrow(newdata))
+    } else {
+      stopifnot(length(offset) %in% c(1L, nrow(newdata)))
+    }
+    pred_sub <- proj_predfun(fits, newdata = newdata)
+    if (family$family %in% fams_neg_linpred()) {
+      pred_sub <- pred_sub - offset
+    } else {
+      pred_sub <- pred_sub + offset
+    }
+    if (transform) {
+      pred_sub <- family$linkinv(pred_sub)
+    }
+    return(pred_sub)
+  }
+
   fetch_data_wrapper <- function(obs = NULL) {
     fetch_data(data, obs = obs)
   }
@@ -1416,14 +1463,6 @@ init_refmodel <- function(object, data, formula, family, ref_predfun = NULL,
   model_data <- extract_model_data(object, newdata = NULL, extract_y = TRUE)
   weights <- model_data$weights
   offset <- model_data$offset
-  if (length(weights) == 0) {
-    stop("The function supplied to argument `extract_model_data` of ",
-         "init_refmodel() must not return a length-zero element `weights`.")
-  }
-  if (length(offset) == 0) {
-    stop("The function supplied to argument `extract_model_data` of ",
-         "init_refmodel() must not return a length-zero element `offset`.")
-  }
   if (family$for_latent) {
     y <- rowMeans(ref_predfun(
       object, excl_offs = FALSE,
@@ -1451,19 +1490,20 @@ init_refmodel <- function(object, data, formula, family, ref_predfun = NULL,
       }
     } else if (family$family_oscale == "binomial") {
       if (!all(is_wholenumber(y_oscale))) {
-        stop(
-          "In projpred, the response must contain numbers of successes (not ",
-          "proportions of successes), in contrast to glm() where this is ",
-          "possible for a 1-column response if the multiplication with the ",
-          "weights gives whole numbers."
-        )
+        # In projpred, the response must contain numbers of successes (not
+        # proportions of successes), in contrast to glm() where this is possible
+        # for a 1-column response if the multiplication with the weights gives
+        # whole numbers:
+        stop("If the original family is the binomial family, the original ",
+             "response values must be numbers of successes (not proportions ",
+             "of successes).")
       } else if (all(y_oscale %in% c(0, 1)) &&
                  length(response_name) == 1 &&
                  !all(weights == 1)) {
-        warning(
-          "Assuming that the response contains numbers of successes (not ",
-          "proportions of successes), in contrast to glm()."
-        )
+        # Assuming that the response contains numbers of successes (not
+        # proportions of successes), in contrast to glm():
+        warning("Assuming that the original response values are numbers of ",
+                "successes (not proportions of successes).")
       }
     }
   } else {
@@ -1489,18 +1529,25 @@ init_refmodel <- function(object, data, formula, family, ref_predfun = NULL,
     }
   } else if (family$family == "binomial") {
     if (!all(is_wholenumber(y))) {
-      stop("In projpred, the response must contain numbers of successes (not ",
-           "proportions of successes), in contrast to glm() where this is ",
-           "possible for a 1-column response if the multiplication with the ",
-           "weights gives whole numbers.")
+      # In projpred, the response must contain numbers of successes (not
+      # proportions of successes), in contrast to glm() where this is possible
+      # for a 1-column response if the multiplication with the weights gives
+      # whole numbers:
+      stop("In case of the binomial family, the response values must be ",
+           "numbers of successes (not proportions of successes).")
     } else if (all(y %in% c(0, 1)) &&
                length(response_name) == 1 &&
                !all(weights == 1)) {
-      warning("Assuming that the response contains numbers of successes (not ",
-              "proportions of successes), in contrast to glm().")
+      # Assuming that the response contains numbers of successes (not
+      # proportions of successes), in contrast to glm():
+      warning("Assuming that the response values are numbers of successes ",
+              "(not proportions of successes).")
     }
   }
 
+  # Already performed by extract_model_data(), but repeated here because
+  # get_standard_y() might have altered the observation weights (in case of a
+  # 2-column `y`):
   if (family$for_augdat && !all(weights == 1)) {
     stop("Currently, the augmented-data projection may not be combined with ",
          "observation weights (other than 1).")
@@ -1613,6 +1660,14 @@ init_refmodel <- function(object, data, formula, family, ref_predfun = NULL,
     }
   } else {
     stopifnot(length(dis) == ndraws)
+    if (!proper_model) {
+      warning(
+        "A non-`NULL` argument `dis` was supplied for a `datafit`. The ",
+        "(internal) default `dis` for `datafit`s is zero (see equations (19) ",
+        "and (20) of Piironen et al., 2020, DOI: 10.1214/20-EJS1711). A ",
+        "non-zero value should only be used if you know what you are doing."
+      )
+    }
   }
   if (getOption("projpred.mlvl_pred_new", FALSE) && warn_allrandom_dis &&
       !all(is.na(dis))) {
@@ -1635,8 +1690,8 @@ init_refmodel <- function(object, data, formula, family, ref_predfun = NULL,
 
   refmodel <- nlist(
     fit = object, formula, div_minimizer, family, eta, mu, mu_offs, dis, y,
-    proj_predfun, fetch_data = fetch_data_wrapper, wobs = weights, wdraws_ref,
-    offset, cvfun, cvfits, extract_model_data, ref_predfun, cvrefbuilder,
+    fetch_data = fetch_data_wrapper, wobs = weights, wdraws_ref, offset, cvfun,
+    cvfits, extract_model_data, ref_predfun, mu_fun, cvrefbuilder,
     y_oscale = y_oscale %||% y, nobs = nrow(data)
   )
   if (proper_model) {
