@@ -4,7 +4,9 @@
 #' variable selection. The search part determines the predictor ranking (also
 #' known as solution path), i.e., the best submodel for each submodel size
 #' (number of predictor terms). The evaluation part determines the predictive
-#' performance of the submodels along the predictor ranking.
+#' performance of the submodels along the predictor ranking. A special method is
+#' [varsel.vsel()] which re-uses the search results from an earlier
+#' [varsel()] (or [cv_varsel()]) run, as illustrated in the main vignette.
 #'
 #' @param object An object of class `refmodel` (returned by [get_refmodel()] or
 #'   [init_refmodel()]) or an object that can be passed to argument `object` of
@@ -16,9 +18,11 @@
 #' @param method The method for the search part. Possible options are
 #'   `"forward"` for forward search and `"L1"` for L1 search. See also section
 #'   "Details" below.
-#' @param refit_prj For the evaluation part, should the submodels along the
-#'   predictor ranking be fitted again (`TRUE`) or should their fits from the
-#'   search part be re-used (`FALSE`)?
+#' @param refit_prj For the evaluation part, should the projections onto the
+#' submodels along the predictor ranking be performed again using `ndraws_pred`
+#' draws or `nclusters_pred` clusters (`TRUE`) or should their projections from
+#' the search part, which used `ndraws` draws or `nclusters` clusters, be re-used
+#' (`FALSE`)?
 #' @param ndraws Number of posterior draws used in the search part. Ignored if
 #'   `nclusters` is not `NULL` or in case of L1 search (because L1 search always
 #'   uses a single cluster). If both (`nclusters` and `ndraws`) are `NULL`, the
@@ -47,20 +51,35 @@
 #'   those predictors have no cost and will therefore be selected first, whereas
 #'   `Inf` means those predictors will never be selected. If `NULL`, then `1` is
 #'   used for each predictor.
-#' @param lambda_min_ratio Only relevant for L1 search. Ratio between the
-#'   smallest and largest lambda in the L1-penalized search. This parameter
-#'   essentially determines how long the search is carried out, i.e., how large
-#'   submodels are explored. No need to change this unless the program gives a
+#' @param search_control A `list` of "control" arguments (i.e., tuning
+#'   parameters) for the search. In case of forward search, these arguments are
+#'   passed to the divergence minimizer (see argument `div_minimizer` of
+#'   [init_refmodel()] as well as section "Draw-wise divergence minimizers" of
+#'   [projpred-package]). In case of forward search, `NULL` causes `...` to be
+#'   used not only for the performance evaluation, but also for the search. In
+#'   case of L1 search, possible arguments are:
+#'   * `lambda_min_ratio`: Ratio between the smallest and largest lambda in the
+#'   L1-penalized search (default: `1e-5`). This parameter essentially
+#'   determines how long the search is carried out, i.e., how large submodels
+#'   are explored. No need to change this unless the program gives a warning
+#'   about this.
+#'   * `nlambda`: Number of values in the lambda grid for L1-penalized search
+#'   (default: `150`). No need to change this unless the program gives a warning
+#'   about this.
+#'   * `thresh`: Convergence threshold when computing the L1 path (default:
+#'   `1e-6`). Usually, there is no need to change this.
+#' @param lambda_min_ratio Deprecated (please use `search_control` instead).
+#'   Only relevant for L1 search. Ratio between the smallest and largest lambda
+#'   in the L1-penalized search. This parameter essentially determines how long
+#'   the search is carried out, i.e., how large submodels are explored. No need
+#'   to change this unless the program gives a warning about this.
+#' @param nlambda Deprecated (please use `search_control` instead). Only
+#'   relevant for L1 search. Number of values in the lambda grid for
+#'   L1-penalized search. No need to change this unless the program gives a
 #'   warning about this.
-#' @param nlambda Only relevant for L1 search. Number of values in the lambda
-#'   grid for L1-penalized search. No need to change this unless the program
-#'   gives a warning about this.
-#' @param thresh Only relevant for L1 search. Convergence threshold when
-#'   computing the L1 path. Usually, there is no need to change this.
-#' @param regul A number giving the amount of ridge regularization when
-#'   projecting onto (i.e., fitting) submodels which are GLMs. Usually there is
-#'   no need for regularization, but sometimes we need to add some
-#'   regularization to avoid numerical problems.
+#' @param thresh Deprecated (please use `search_control` instead). Only relevant
+#'   for L1 search. Convergence threshold when computing the L1 path. Usually,
+#'   there is no need to change this.
 #' @param search_terms Only relevant for forward search. A custom character
 #'   vector of predictor term blocks to consider for the search. Section
 #'   "Details" below describes more precisely what "predictor term block" means.
@@ -68,8 +87,14 @@
 #'   there's no difference between including it explicitly or omitting it. The
 #'   default `search_terms` considers all the terms in the reference model's
 #'   formula.
-#' @param verbose A single logical value indicating whether to print out
-#'   additional information during the computations.
+#' @param search_out Intended for internal use.
+#' @param verbose A single integer value from the set \eqn{\{0, 1, 2, 3,
+#'   4\}}{{0, 1, 2, 3, 4}} (for [varsel()], \eqn{3} and \eqn{4} have the same
+#'   effect), indicating how much information (if any) to print out during the
+#'   computations. Higher values indicate that more information should be
+#'   printed, `0` deactivates the verbose mode. Internally, argument `verbose`
+#'   is coerced to integer via `as.integer()`, so technically, a single logical
+#'   value or a single numeric value work as well.
 #' @param seed Pseudorandom number generation (PRNG) seed by which the same
 #'   results can be obtained again if needed. Passed to argument `seed` of
 #'   [set.seed()], but can also be `NA` to not call [set.seed()] at all. If not
@@ -78,9 +103,13 @@
 #'   model's posterior draws (if `!is.null(nclusters)` or
 #'   `!is.null(nclusters_pred)`) and for drawing new group-level effects when
 #'   predicting from a multilevel submodel (however, not yet in case of a GAMM).
-#' @param ... Arguments passed to [get_refmodel()] as well as to the divergence
-#'   minimizer (during a forward search and also during the evaluation part, but
-#'   the latter only if `refit_prj` is `TRUE`).
+#' @param ... For [varsel.default()]: Arguments passed to [get_refmodel()] as
+#'   well as to [varsel.refmodel()]. For [varsel.vsel()]: Arguments passed to
+#'   [varsel.refmodel()]. For [varsel.refmodel()]: Arguments passed to the
+#'   divergence minimizer (see argument `div_minimizer` of [init_refmodel()] as
+#'   well as section "Draw-wise divergence minimizers" of [projpred-package])
+#'   when refitting the submodels for the performance evaluation (if `refit_prj`
+#'   is `TRUE`).
 #'
 #' @details
 #'
@@ -117,8 +146,9 @@
 #'
 #'   L1 search is faster than forward search, but forward search may be more
 #'   accurate. Furthermore, forward search may find a sparser model with
-#'   comparable performance to that found by L1 search, but it may also start
-#'   overfitting when more predictors are added.
+#'   comparable performance to that found by L1 search, but it may also
+#'   overfit when more predictors are added. This overfit can be detected
+#'   by running search validation (see [cv_varsel()]).
 #'
 #'   An L1 search may select an interaction term before all involved lower-order
 #'   interaction terms (including main-effect terms) have been selected. In
@@ -166,7 +196,7 @@
 #' # Data:
 #' dat_gauss <- data.frame(y = df_gaussian$y, df_gaussian$x)
 #'
-#' # The "stanreg" fit which will be used as the reference model (with small
+#' # The `stanreg` fit which will be used as the reference model (with small
 #' # values for `chains` and `iter`, but only for technical reasons in this
 #' # example; this is not recommended in general):
 #' fit <- rstanarm::stan_glm(
@@ -197,25 +227,73 @@ varsel.default <- function(object, ...) {
 #' @rdname varsel
 #' @export
 varsel.vsel <- function(object, ...) {
-  stop("Purpose and content of varsel.vsel() will be changed in a future ",
-       "release. Please use varsel(get_refmodel(<vsel_object>), <...>) ",
-       "instead of varsel(<vsel_object>, <...>).")
+  arg_nms_internal <- c("method", "ndraws", "nclusters", "nterms_max",
+                        "search_control", "penalty", "search_terms")
+  arg_nms_internal_used <- intersect(arg_nms_internal, ...names())
+  n_arg_nms_internal_used <- length(arg_nms_internal_used)
+  if (n_arg_nms_internal_used > 0) {
+    stop("Argument", if (n_arg_nms_internal_used > 1) "s" else "", " ",
+         paste(paste0("`", arg_nms_internal_used, "`"), collapse = ", "), " ",
+         "cannot be specified in this case because varsel.vsel() specifies ",
+         if (n_arg_nms_internal_used > 1) "them" else "it", " ", "internally.")
+  }
+  return(varsel(
+    object = get_refmodel(object),
+    method = object[["args_search"]][["method"]],
+    ndraws = object[["args_search"]][["ndraws"]],
+    nclusters = object[["args_search"]][["nclusters"]],
+    nterms_max = object[["args_search"]][["nterms_max"]],
+    search_control = object[["args_search"]][["search_control"]],
+    penalty = object[["args_search"]][["penalty"]],
+    search_terms = object[["args_search"]][["search_terms"]],
+    search_out = list(search_path = object[["search_path"]]),
+    ...
+  ))
 }
 
 #' @rdname varsel
 #' @export
-varsel.refmodel <- function(object, d_test = NULL, method = "forward",
-                            ndraws = NULL, nclusters = 20, ndraws_pred = 400,
-                            nclusters_pred = NULL,
-                            refit_prj = !inherits(object, "datafit"),
-                            nterms_max = NULL, verbose = TRUE,
-                            lambda_min_ratio = 1e-5, nlambda = 150,
-                            thresh = 1e-6, regul = 1e-4, penalty = NULL,
-                            search_terms = NULL, seed = NA, ...) {
-  if (missing(method) && getOption("projpred.mssg_method_changed", TRUE)) {
-    message("NOTE: In projpred 2.7.0, the default search method ",
-            "was set to \"forward\" for all kinds of models.")
+varsel.refmodel <- function(
+    object,
+    d_test = NULL,
+    method = "forward",
+    ndraws = NULL,
+    nclusters = 20,
+    ndraws_pred = 400,
+    nclusters_pred = NULL,
+    refit_prj = !inherits(object, "datafit"),
+    nterms_max = NULL,
+    verbose = getOption("projpred.verbose", as.integer(interactive())),
+    search_control = NULL,
+    lambda_min_ratio = 1e-5,
+    nlambda = 150,
+    thresh = 1e-6,
+    penalty = NULL,
+    search_terms = NULL,
+    search_out = NULL,
+    seed = NA,
+    ...
+) {
+  if (!missing(lambda_min_ratio)) {
+    warning("Argument `lambda_min_ratio` is deprecated. Please specify ",
+            "control arguments for the search via argument `search_control`. ",
+            "Now using `lambda_min_ratio` as element `lambda_min_ratio` of ",
+            "`search_control`.")
+    search_control$lambda_min_ratio <- lambda_min_ratio
   }
+  if (!missing(nlambda)) {
+    warning("Argument `nlambda` is deprecated. Please specify control ",
+            "arguments for the search via argument `search_control`. ",
+            "Now using `nlambda` as element `nlambda` of `search_control`.")
+    search_control$nlambda <- nlambda
+  }
+  if (!missing(thresh)) {
+    warning("Argument `thresh` is deprecated. Please specify control ",
+            "arguments for the search via argument `search_control`. ",
+            "Now using `thresh` as element `thresh` of `search_control`.")
+    search_control$thresh <- thresh
+  }
+  verbose <- verbose_from_deprecated_options(verbose, with_cv = FALSE)
 
   if (exists(".Random.seed", envir = .GlobalEnv)) {
     rng_state_old <- get(".Random.seed", envir = .GlobalEnv)
@@ -235,7 +313,7 @@ varsel.refmodel <- function(object, d_test = NULL, method = "forward",
   args <- parse_args_varsel(
     refmodel = refmodel, method = method, refit_prj = refit_prj,
     nterms_max = nterms_max, nclusters = nclusters, search_terms = search_terms,
-    nterms_all = nterms_all
+    nterms_all = nterms_all, verbose = verbose
   )
   method <- args$method
   refit_prj <- args$refit_prj
@@ -243,6 +321,7 @@ varsel.refmodel <- function(object, d_test = NULL, method = "forward",
   nclusters <- args$nclusters
   search_terms <- args$search_terms
   search_terms_was_null <- args$search_terms_was_null
+  verbose <- args$verbose
 
   # Pre-process `d_test`:
   if (is.null(d_test)) {
@@ -257,6 +336,22 @@ varsel.refmodel <- function(object, d_test = NULL, method = "forward",
     d_test <- d_test[nms_d_test()]
     invisible(lapply(setNames(nm = setdiff(nms_d_test(), c("y", "y_oscale"))),
                      function(d_nm) na.fail(d_test[[d_nm]])))
+    hasNA_y_test <- is.na(d_test[["y"]])
+    if (any(hasNA_y_test)) {
+      stopifnot(all(hasNA_y_test))
+    }
+    hasNA_y_oscale_test <- is.na(d_test[["y_oscale"]])
+    if (any(hasNA_y_oscale_test)) {
+      stopifnot(all(hasNA_y_oscale_test))
+    }
+    if (length(d_test[["weights"]]) != nrow(d_test[["data"]])) {
+      stop("Element `d_test$weights` needs to have length equal to the number ",
+           "of test observations.")
+    }
+    if (length(d_test[["offset"]]) != nrow(d_test[["data"]])) {
+      stop("Element `d_test$offset` needs to have length equal to the number ",
+           "of test observations.")
+    }
     if (refmodel$family$for_augdat) {
       d_test$y <- as.factor(d_test$y)
       if (!all(levels(d_test$y) %in% refmodel$family$cats)) {
@@ -296,29 +391,28 @@ varsel.refmodel <- function(object, d_test = NULL, method = "forward",
   nobs_test <- nrow(y_wobs_test)
 
   # Run the search:
-  opt <- nlist(lambda_min_ratio, nlambda, thresh, regul)
-  verb_out("-----\nRunning the search ...", verbose = verbose)
-  search_path <- select(
-    refmodel = refmodel, ndraws = ndraws, nclusters = nclusters,
-    method = method, nterms_max = nterms_max, penalty = penalty,
-    verbose = verbose, opt = opt, search_terms = search_terms,
-    search_terms_was_null = search_terms_was_null, ...
-  )
-  verb_out("-----", verbose = verbose)
+  if (!is.null(search_out)) {
+    search_path <- search_out[["search_path"]]
+  } else {
+    search_path <- .select(
+      refmodel = refmodel, ndraws = ndraws, nclusters = nclusters,
+      method = method, nterms_max = nterms_max, penalty = penalty,
+      verbose = verbose, search_control = search_control,
+      search_terms = search_terms,
+      search_terms_was_null = search_terms_was_null, ...
+    )
+  }
 
   # "Run" the performance evaluation for the submodels along the predictor
   # ranking (in fact, we only prepare the performance evaluation by computing
   # precursor quantities, but for users, this difference is not perceivable):
-  verb_out("-----\nRunning the performance evaluation ...",
-           verbose = verbose && refit_prj)
   perf_eval_out <- perf_eval(
-    search_path = search_path, refmodel = refmodel, regul = regul,
-    refit_prj = refit_prj, ndraws = ndraws_pred, nclusters = nclusters_pred,
-    indices_test = NULL, newdata_test = d_test$data,
-    offset_test = d_test$offset, wobs_test = d_test$weights, y_test = d_test$y,
-    y_oscale_test = d_test$y_oscale, ...
+    search_path = search_path, refmodel = refmodel, refit_prj = refit_prj,
+    ndraws = ndraws_pred, nclusters = nclusters_pred, indices_test = NULL,
+    newdata_test = d_test$data, offset_test = d_test$offset,
+    wobs_test = d_test$weights, y_test = d_test$y,
+    y_oscale_test = d_test$y_oscale, verbose = verbose, ...
   )
-  verb_out("-----", verbose = verbose && refit_prj)
 
   # Predictive performance of the reference model:
   if (inherits(refmodel, "datafit")) {
@@ -332,7 +426,7 @@ varsel.refmodel <- function(object, d_test = NULL, method = "forward",
       if (!is.null(refmodel$family$cats)) {
         mu_oscale <- structure(rep(NA,
                                    nobs_test * length(refmodel$family$cats)),
-                               nobs_orig = nobs_test,
+                               ndiscrete = length(refmodel$family$cats),
                                class = "augvec")
       } else {
         mu_oscale <- ref$mu
@@ -376,23 +470,41 @@ varsel.refmodel <- function(object, d_test = NULL, method = "forward",
   vs <- nlist(refmodel,
               nobs_train = refmodel$nobs,
               search_path,
-              solution_terms = search_path$solution_terms,
-              solution_terms_cv = NULL,
+              predictor_ranking = search_path$predictor_ranking,
+              predictor_ranking_cv = NULL,
               ce = perf_eval_out[["ce"]],
               type_test = d_test$type,
               y_wobs_test,
               nobs_test,
               summaries = nlist(sub = perf_eval_out[["sub_summaries"]], ref),
+              summaries_fast = NULL,
               nterms_all,
               nterms_max,
               method,
               cv_method = NULL,
+              nloo = NULL,
+              loo_inds = NULL,
               K = NULL,
               validate_search = NULL,
+              ### Not set to `NULL` because in K-fold CV (relevant when using
+              ### cv_varsel.vsel() based on this `vsel` object), `cvfits = NULL`
+              ### would mean to use `cvfun`; instead, the default should be
+              ### element `cvfits` from `refmodel`, so:
+              cvfits = refmodel$cvfits,
+              ###
+              args_search = nlist(
+                method, ndraws, nclusters, nterms_max,
+                search_control = if (
+                  method == "forward" && is.null(search_control)
+                ) list(...) else search_control,
+                penalty,
+                search_terms = if (search_terms_was_null) NULL else search_terms
+              ),
               clust_used_search = search_path$p_sel$clust_used,
               clust_used_eval = perf_eval_out[["clust_used"]],
-              nprjdraws_search = NCOL(search_path$p_sel$mu),
+              nprjdraws_search = search_path$p_sel$nprjdraws,
               nprjdraws_eval = perf_eval_out[["nprjdraws"]],
+              refit_prj,
               projpred_version = utils::packageVersion("projpred"))
   class(vs) <- "vsel"
 
@@ -406,14 +518,19 @@ varsel.refmodel <- function(object, d_test = NULL, method = "forward",
 #   weights), then this needs to be a `list` with elements `wdraws_ref` and
 #   `cl_ref`. For these two elements, see the (internal) documentation of
 #   weighted_summary_means().
+# @param verbose_txt_add Passed to `...` of verb_out(), so character string(s)
+#   to be included in the verbose message indicating the start of the search.
 # For all other arguments, see the documentation of varsel().
 #
-# @return A list with elements `solution_terms` (the solution path), `outdmins`
-#   (the submodel fits along the solution path, with the number of fits per
-#   model size being equal to the number of projected draws), and `p_sel` (the
-#   output from get_refdist() for the search).
-select <- function(refmodel, ndraws, nclusters, reweighting_args = NULL, method,
-                   nterms_max, penalty, verbose, opt, ...) {
+# @return A list with elements `predictor_ranking` (the predictor ranking
+#   resulting from the search, see `?ranking` for a formal definition),
+#   `outdmins` (the submodel fits along the predictor ranking, with the number
+#   of fits per model size being equal to the number of projected draws), and
+#   `p_sel` (the output from get_refdist() for the search).
+.select <- function(refmodel, ndraws, nclusters, reweighting_args = NULL,
+                    method, nterms_max, penalty, verbose,
+                    verbose_line_length = 5, verbose_txt_add = "",
+                    search_control, ...) {
   if (is.null(reweighting_args)) {
     p_sel <- get_refdist(refmodel, ndraws = ndraws, nclusters = nclusters)
   } else {
@@ -421,20 +538,32 @@ select <- function(refmodel, ndraws, nclusters, reweighting_args = NULL, method,
     p_sel <- get_p_clust(
       family = refmodel$family, eta = refmodel$eta, mu = refmodel$mu,
       mu_offs = refmodel$mu_offs, dis = refmodel$dis,
-      wdraws = reweighting_args$wdraws_ref, cl = reweighting_args$cl_ref
+      wdraws = reweighting_args$wdraws_ref, cl = reweighting_args$cl_ref,
+      clust_used_forced = clust_info(
+        ndraws = ndraws,
+        nclusters = nclusters,
+        S = length(refmodel$wdraws_ref)
+      )[["clust_used"]]
     )
   }
+
+  verb_out(rep("-", verbose_line_length), "\nRunning ", method, " search ",
+           verbose_txt_add, "with ",
+           txt_clust_draws(p_sel[["clust_used"]], p_sel[["nprjdraws"]]),
+           " ...", verbose = verbose)
   if (method == "L1") {
     search_path <- search_L1(
       p_ref = p_sel, refmodel = refmodel, nterms_max = nterms_max,
-      penalty = penalty, opt = opt
+      penalty = penalty, search_control = search_control
     )
   } else if (method == "forward") {
     search_path <- search_forward(
       p_ref = p_sel, refmodel = refmodel, nterms_max = nterms_max,
-      verbose = verbose, opt = opt, ...
+      verbose = verbose, search_control = search_control, ...
     )
   }
+  verb_out(rep("-", verbose_line_length), verbose = verbose)
+
   search_path$p_sel <- p_sel
   return(search_path)
 }
@@ -446,7 +575,7 @@ select <- function(refmodel, ndraws, nclusters, reweighting_args = NULL, method,
 # them in with the default values. The purpose of this function is to avoid
 # repeating the same code both in varsel() and cv_varsel().
 parse_args_varsel <- function(refmodel, method, refit_prj, nterms_max,
-                              nclusters, search_terms, nterms_all) {
+                              nclusters, search_terms, nterms_all, verbose) {
   search_terms_was_null <- is.null(search_terms)
   if (search_terms_was_null) {
     search_terms <- split_formula(refmodel$formula,
@@ -480,8 +609,8 @@ parse_args_varsel <- function(refmodel, method, refit_prj, nterms_max,
 
   stopifnot(!is.null(refit_prj))
   if (refit_prj && inherits(refmodel, "datafit")) {
-    warning("For an `object` of class \"datafit\", `refit_prj` is ",
-            "automatically set to `FALSE`.")
+    warning("For an `object` of class `datafit`, `refit_prj` is automatically ",
+            "set to `FALSE`.")
     refit_prj <- FALSE
   }
 
@@ -528,6 +657,8 @@ parse_args_varsel <- function(refmodel, method, refit_prj, nterms_max,
     )
   }
 
+  verbose <- as.integer(verbose)
+
   return(nlist(method, refit_prj, nterms_max, nclusters, search_terms,
-               search_terms_was_null))
+               search_terms_was_null, verbose))
 }
