@@ -201,7 +201,7 @@ List glm_elnet_c(arma::mat x, // input matrix
   
   obs = pseudo_obs(f,w0);
   z = as<vec>(obs["z"]);
-  w = as<vec>(obs["w"]);
+  w = as<vec>(obs["wobs"]);
   double loss_initial = loss_approx(beta, f, z, w, lambda(0), alpha, penalty); // initial loss
   double loss_old = loss_initial; // will be updated iteratively
   double loss; // will be updated iteratively
@@ -218,7 +218,7 @@ List glm_elnet_c(arma::mat x, // input matrix
       // update the quadratic likelihood approximation
       obs = pseudo_obs(f,w);
       z = as<vec>(obs["z"]);
-      w = as<vec>(obs["w"]);
+      w = as<vec>(obs["wobs"]);
       ++qau;
       
       // current value of the (approximate) loss function
@@ -336,7 +336,7 @@ void glm_ridge( vec& beta,      // output: regression coefficients (contains int
   // initial quadratic approximation
   List obs = pseudo_obs(f,w);
   vec z = as<vec>(obs["z"]);
-  w = as<vec>(obs["w"]);
+  w = as<vec>(obs["wobs"]);
   grad_f = as<vec>(obs["grad"]);
   double loss_initial = ((double) obs["loss"]) + elnet_penalty(beta, lambda, 0, penalty);
   double loss_old = loss_initial; // will be updated iteratively
@@ -403,7 +403,7 @@ void glm_ridge( vec& beta,      // output: regression coefficients (contains int
     // update the solution
     beta = beta + t*dbeta;
     z = as<vec>(obs["z"]);
-    w = as<vec>(obs["w"]);
+    w = as<vec>(obs["wobs"]);
     grad_f = as<vec>(obs["grad"]);
     loss_old = loss;
     ++qau;
@@ -456,87 +456,3 @@ List glm_ridge_c( arma::mat x,
 }
 
 
-
-
-
-
-/** Forward search for glm.
- */
-// [[Rcpp::export]]
-List glm_forward_c( arma::mat x, // inputs (features)
-                    Function pseudo_obs, // R-function returning the pseudo-data based on the quadratic approximation
-                    double lambda, // regularization parameter (multiplier for L2-penalty)
-                    bool intercept, // whether to use intercept
-                    arma::vec penalty, // relative penalties for the variables
-                    double thresh, // threshold for stopping the iterative reweighted least squares
-                    int qa_updates_max, // max number or quadratic approximation updates
-                    int pmax, // maximum number of variables up to which the search is continued
-                    arma::vec w0, // initial guess for the weights of the pseudo-gaussian observations (needed for Student-t model)
-                    int ls_iter_max=50 ) // max number of line search iterations
-{
-  
-  mat xp; // x for the current active set
-  mat xp_temp; // x for the current active set + the variable to be added
-  size_t D = x.n_cols; // total number of inputs
-  size_t pmaxu = (size_t) pmax; // converting pmax to unsigned int (avoids some compiler warnings)
-  uvec chosen(D); chosen.zeros(); // keeps track of added variables
-  uvec varorder(pmaxu); varorder.zeros(); // stores the order in which the variables are added to the model
-  mat beta_all(D,pmaxu); beta_all.zeros(); // collects beta from all steps
-  rowvec beta0_all(pmaxu); beta0_all.zeros(); // collects beta0 from all steps
-  mat w_all(x.n_rows,pmaxu); // collects weights of the gaussian pseudo-observations from all steps
-  
-  // declare a few variables that are needed during the iteration
-  vec w = w0;
-  int qau;
-  size_t j,k,jopt=0;
-  uvec varind;
-  uvec step(1);
-  
-  for (k=0; k<pmaxu; ++k) {
-    
-    varind = find(chosen);
-    vec beta(varind.size() + 1 + intercept);
-    vec betaopt;
-    double loss_min = std::numeric_limits<double>::infinity();
-    double loss;
-    
-    // loop through all the candidate variables that could be added next
-    for (j=0; j<D; ++j) {
-    
-      if (chosen(j))
-        continue;
-      
-      chosen(j) = 1;
-      beta.zeros();
-      glm_ridge(beta, loss, w, qau, x.cols(find(chosen)), pseudo_obs, lambda, intercept, 
-                penalty.elem(find(chosen)), thresh, qa_updates_max, ls_iter_max);
-      chosen(j) = 0;
-      
-      if (loss < loss_min) {
-        loss_min = loss;
-        jopt = j;
-        betaopt = beta;
-      }
-    }
-    
-    varorder(k) = jopt;
-    chosen(jopt) = 1;
-    step(0) = k;
-    
-    if (intercept) {
-      beta0_all(k) = betaopt(0);
-      beta_all.submat(find(chosen), step) = betaopt.tail(k+1);
-    } else {
-      beta0_all(k) = 0;
-      beta_all.submat(find(chosen), step) = betaopt;
-    }
-    w_all.col(k) = w;
-  }
-  
-  return List::create(beta_all, beta0_all, varorder, w_all);
-}
-
-
-
-
-
