@@ -9,19 +9,13 @@
 #   next sublist is unnamed or contains an element with name given in
 #   `nm_stop`).
 unlist_cust <- function(x, nm_stop = "mod_nm") {
-  stopifnot(is.list(x) && length(x) > 0)
+  stopifnot(is.list(x))
+  if (!length(x)) {
+    return(x)
+  }
   if (is.null(names(x[[1]])) || nm_stop %in% names(x[[1]])) {
     return(x)
   } else {
     return(unlist_cust(unlist(x, recursive = FALSE), nm_stop = nm_stop))
   }
 }
-
-# Remove undesired "dummy" entries from an "unlisted" (via unlist_cust()) list.
-#
-# @param x_unlisted A list returned by unlist_cust().
-#
-# @return `x_unlisted` with dummy elements removed..
-rm_dummies_unlisted <- function(x_unlisted) {
-  return(x_unlisted[!sapply(x_unlisted, identical, "REMOVE THIS DUMMY ENTRY")])
-}
