@@ -18,17 +18,14 @@ augmtst <- arr2augmat(arrtst)
 
 test_that("arr2augmat(), augmat2arr()", {
   expect_identical(augmtst, structure(matrix(xtst, ncol = ndraws_tst),
-                                      nobs_orig = nobs_orig_tst,
-                                      class = "augmat"))
+                                      ndiscrete = ncat_tst, class = "augmat"))
 
   # With "margin_draws = 3":
   xtst_m3 <- 0.1
   stopifnot(length(xtst_m3) == 1)
   arr_m3 <- array(xtst_m3, dim = c(1, 1, 1))
   augmat_m3 <- arr2augmat(arr_m3)
-  augmat_m3_ch <- structure(matrix(xtst_m3),
-                            nobs_orig = 1L,
-                            class = "augmat")
+  augmat_m3_ch <- structure(matrix(xtst_m3), ndiscrete = 1L, class = "augmat")
   expect_identical(augmat_m3, augmat_m3_ch)
   arr_m3_ch <- augmat2arr(augmat_m3)
   expect_identical(arr_m3_ch, arr_m3)
@@ -39,8 +36,7 @@ test_that("arr2augmat(), augmat2arr()", {
   arr_m1 <- array(xtst_m1, dim = c(7, 3, 1))
   augmat_m1 <- arr2augmat(arr_m1, margin_draws = 1)
   augmat_m1_ch <- structure(matrix(xtst_m1, ncol = 7, byrow = TRUE),
-                            nobs_orig = 3L,
-                            class = "augmat")
+                            ndiscrete = 1L, class = "augmat")
   expect_identical(augmat_m1, augmat_m1_ch)
   arr_m1_ch <- augmat2arr(augmat_m1, margin_draws = 1)
   expect_identical(arr_m1_ch, arr_m1)
@@ -62,16 +58,13 @@ test_that("`[.augmat`()", {
   expect_identical(augmtst[, 1:2],
                    structure(matrix(head(xtst, nobs_orig_tst * ncat_tst * 2),
                                     ncol = 2),
-                             nobs_orig = nobs_orig_tst,
-                             class = "augmat"))
+                             ndiscrete = ncat_tst, class = "augmat"))
   expect_identical(augmtst[, 1, drop = FALSE],
                    structure(matrix(head(xtst, nobs_orig_tst * ncat_tst)),
-                             nobs_orig = nobs_orig_tst,
-                             class = "augmat"))
+                             ndiscrete = ncat_tst, class = "augmat"))
   expect_identical(augmtst[, 1],
                    structure(head(xtst, nobs_orig_tst * ncat_tst),
-                             nobs_orig = nobs_orig_tst,
-                             class = "augvec"))
+                             ndiscrete = ncat_tst, class = "augvec"))
   # Subsetting rows:
   xrows1 <- xtst[as.vector(t(sapply(
     nobs_orig_tst * (seq_len(ncat_tst) - 1L) + 1L,
@@ -82,32 +75,42 @@ test_that("`[.augmat`()", {
   xrows1 <- matrix(xrows1, nrow = ncat_tst)
   expect_identical(augmtst[nobs_orig_tst * (seq_len(ncat_tst) - 1L) + 1L, ,
                            drop = FALSE],
-                   structure(xrows1,
-                             nobs_orig = 1L,
-                             class = "augmat"))
-  expect_error(augmtst[1, ], "is_wholenumber.+ is not TRUE")
-  expect_error(augmtst[1, , drop = FALSE], "is_wholenumber.+ is not TRUE")
+                   structure(xrows1, ndiscrete = ncat_tst, class = "augmat"))
+  # "Illegally" subsetting not only observations, but also the (possibly latent)
+  # response categories (within projpred, this should never be used, but testing
+  # here anyway for the sake of completeness):
+  xrow1 <- xtst[nobs_orig_tst * ncat_tst * (seq_len(ndraws_tst) - 1L) + 1L]
+  expect_identical(augmtst[1, ],
+                   structure(xrow1, ndiscrete = ncat_tst, class = "augvec"))
+  expect_identical(augmtst[1, , drop = FALSE],
+                   structure(t(xrow1), ndiscrete = ncat_tst, class = "augmat"))
   # Subsetting rows and columns:
   expect_identical(
     augmtst[sort(c(nobs_orig_tst * (seq_len(ncat_tst) - 1L) + 1L,
                    nobs_orig_tst * (seq_len(ncat_tst) - 1L) + 2L)), 1:2],
     structure(cbind(as.vector(arrtst[1:2, , 1]), as.vector(arrtst[1:2, , 2])),
-              nobs_orig = 2L,
-              class = "augmat")
+              ndiscrete = ncat_tst, class = "augmat")
   )
   expect_identical(augmtst[nobs_orig_tst * (seq_len(ncat_tst) - 1L) + 1L, 1],
-                   structure(xrows1[, 1],
-                             nobs_orig = 1L,
+                   structure(xrows1[, 1], ndiscrete = ncat_tst,
                              class = "augvec"))
-  expect_error(augmtst[1:3, 1:2], "is_wholenumber.+ is not TRUE")
-  expect_error(augmtst[1, 1], "is_wholenumber.+ is not TRUE")
-  expect_error(augmtst[1, 1, drop = FALSE], "is_wholenumber.+ is not TRUE")
+  # "Illegally" subsetting not only observations, but also the (possibly latent)
+  # response categories (within projpred, this should never be used, but testing
+  # here anyway for the sake of completeness):
+  expect_identical(augmtst[1:3, 1:2],
+                   structure(arrtst[1:3, 1, 1:2], ndiscrete = ncat_tst,
+                             class = "augmat"))
+  expect_identical(augmtst[1, 1],
+                   structure(head(xtst, 1), ndiscrete = ncat_tst,
+                             class = "augvec"))
+  expect_identical(augmtst[1, 1, drop = FALSE],
+                   structure(matrix(head(xtst, 1)), ndiscrete = ncat_tst,
+                             class = "augmat"))
 
   # Assigning:
   augmtst[1, 1] <- 0
   expect_identical(augmtst, structure(matrix(c(0, xtst[-1]), ncol = ndraws_tst),
-                                      nobs_orig = nobs_orig_tst,
-                                      class = "augmat"))
+                                      ndiscrete = ncat_tst, class = "augmat"))
 })
 
 ## augmat2augvec(), augvec2augmat() ---------------------------------------
@@ -136,21 +139,21 @@ test_that("`[.augvec`()", {
   expect_identical(augvtst[], augvtst)
   xrows1_1 <- xtst[nobs_orig_tst * (seq_len(ncat_tst) - 1L) + 1L]
   expect_identical(augvtst[nobs_orig_tst * (seq_len(ncat_tst) - 1L) + 1L],
-                   structure(xrows1_1,
-                             nobs_orig = 1L,
+                   structure(xrows1_1, ndiscrete = ncat_tst, class = "augvec"))
+  expect_identical(augvtst[1:2],
+                   structure(head(xtst, 2), ndiscrete = ncat_tst,
+                             class = "augvec"))
+  expect_identical(augvtst[1],
+                   structure(head(xtst, 1), ndiscrete = ncat_tst,
                              class = "augvec"))
-  expect_error(augvtst[1], "is_wholenumber.+ is not TRUE")
   expect_identical(augvtst[integer()],
-                   structure(integer(),
-                             nobs_orig = 0L,
-                             class = "augvec"))
+                   structure(integer(), ndiscrete = ncat_tst, class = "augvec"))
 
   # Assigning:
   augvtst[1] <- 0
   expect_identical(augvtst,
                    structure(c(0, head(xtst, nobs_orig_tst * ncat_tst)[-1]),
-                             nobs_orig = nobs_orig_tst,
-                             class = "augvec"))
+                             ndiscrete = ncat_tst, class = "augvec"))
 })
 
 ## catmaxprb() ------------------------------------------------------------
@@ -161,9 +164,7 @@ test_that("catmaxprb()", {
                    factor(rep("1", nobs_orig_tst), levels = c("0", "1")))
 
   # With probabilities (and a single observation):
-  augvtst_pr <- structure(c(0.7, 0.3),
-                          nobs_orig = 1L,
-                          class = "augvec")
+  augvtst_pr <- structure(c(0.7, 0.3), ndiscrete = 2L, class = "augvec")
   expect_identical(catmaxprb(augvtst_pr, lvls = c("lvl1", "lvl2")),
                    factor("lvl1", levels = c("lvl1", "lvl2")))
 })
@@ -210,7 +211,7 @@ test_that(paste(
 
     eta_aug <- refmod_crr$family$linkfun(pref_aug$mu)
     eta_trad <- refmod_crr_trad$family$linkfun(pref_trad$mu)
-    expect_identical(structure(unclass(eta_aug), nobs_orig = NULL), eta_trad,
+    expect_identical(structure(unclass(eta_aug), ndiscrete = NULL), eta_trad,
                      info = tstsetup)
   }
   if (exists("rng_old")) assign(".Random.seed", rng_old, envir = .GlobalEnv)
@@ -229,14 +230,15 @@ test_that(paste(
     tstsetup_trad <- sub("\\.augdat\\.", ".trad_compare.", tstsetup)
     if (!tstsetup_trad %in% names(prjs)) next
 
-    cl_used <- ndr_ncl_dtls(args_prj[[tstsetup]])$clust_used
-    prjmat <- as.matrix(prjs[[tstsetup]], allow_nonconst_wdraws_prj = cl_used)
+    cl_used_gt1 <- ndr_ncl_dtls(args_prj[[tstsetup]])$clust_used_gt1
+    prjmat <- as.matrix(prjs[[tstsetup]],
+                        allow_nonconst_wdraws_prj = cl_used_gt1)
     prjmat_trad <- as.matrix(prjs[[tstsetup_trad]],
-                             allow_nonconst_wdraws_prj = cl_used)
+                             allow_nonconst_wdraws_prj = cl_used_gt1)
 
     tol_coefs <- ifelse(
       args_prj[[tstsetup]]$mod_nm == "glmm" &&
-        any(grepl("^\\(.*\\)$", args_prj[[tstsetup]]$solution_terms)),
+        any(grepl("^\\(.*\\)$", args_prj[[tstsetup]]$predictor_terms)),
       1e-3, 1e-5
     )
     expect_equal(prjmat, prjmat_trad, tolerance = tol_coefs, info = tstsetup)
@@ -265,11 +267,11 @@ test_that(paste(
       list(object = refmods[[args_prj_i_trad$tstsetup_ref]], regul = 0),
       excl_nonargs(args_prj_i_trad)
     ))
-    cl_used <- ndr_ncl_dtls(args_prj_i)$clust_used
-    prjmat <- as.matrix(prj, allow_nonconst_wdraws_prj = cl_used)
-    prjmat_trad <- as.matrix(prj_trad, allow_nonconst_wdraws_prj = cl_used)
+    cl_used_gt1 <- ndr_ncl_dtls(args_prj_i)$clust_used_gt1
+    prjmat <- as.matrix(prj, allow_nonconst_wdraws_prj = cl_used_gt1)
+    prjmat_trad <- as.matrix(prj_trad, allow_nonconst_wdraws_prj = cl_used_gt1)
 
-    tol_coefs <- ifelse(cl_used, 1e-9, 1e-14)
+    tol_coefs <- ifelse(cl_used_gt1, 1e-9, 1e-14)
     expect_equal(prjmat, prjmat_trad, tolerance = tol_coefs, info = tstsetup)
   }
 })
@@ -300,7 +302,7 @@ test_that(paste(
 
     tol_lpreds <- ifelse(
       args_prj[[tstsetup]]$mod_nm == "glmm" &&
-        any(grepl("^\\(.*\\)$", args_prj[[tstsetup]]$solution_terms)),
+        any(grepl("^\\(.*\\)$", args_prj[[tstsetup]]$predictor_terms)),
       1e-4, 1e-5
     )
     expect_equal(pl, pl_trad, tolerance = tol_lpreds, info = tstsetup)
@@ -338,10 +340,12 @@ test_that(paste(
     expect_equal(summs_sub_lppd, summs_sub_lppd_trad, tolerance = 1e-6,
                  info = tstsetup)
     summs_ref <- vs$summaries$ref
-    summs_ref$mu <- structure(unclass(summs_ref$mu), nobs_orig = NULL)
+    summs_ref$mu <- structure(unclass(summs_ref$mu), ndiscrete = NULL)
     summs_ref$mu <- summs_ref$mu[(nobsv + 1):(2 * nobsv)]
-    tol_ref <- .Machine$double.eps
-    if (args_vs[[tstsetup]]$mod_nm == "glmm") {
+    tol_ref <- 1e1 * .Machine$double.eps
+    if (args_vs[[tstsetup]]$mod_nm == "glmm" ||
+        (args_vs[[tstsetup]]$mod_nm == "glm" &&
+         args_vs[[tstsetup]]$fam_nm %in% c("brnll", "binom"))) {
       tol_ref <- 1e3 * tol_ref
     }
     expect_equal(summs_ref, vs_trad$summaries$ref, tolerance = tol_ref,
@@ -365,16 +369,16 @@ test_that(paste(
     smmry_vs_trad <- smmrys_vs[[tstsetup_trad]]
 
     expect_equal(
-      smmry_vs[setdiff(names(smmry_vs), c("family", "selection"))],
-      smmry_vs_trad[setdiff(names(smmry_vs_trad), c("family", "selection"))],
+      smmry_vs[setdiff(names(smmry_vs), c("family", "perf_sub"))],
+      smmry_vs_trad[setdiff(names(smmry_vs_trad), c("family", "perf_sub"))],
       info = tstsetup
     )
     expect_identical(
-      smmry_vs$selection[, c("size", "solution_terms")],
-      smmry_vs_trad$selection[, c("size", "solution_terms")],
+      smmry_vs$perf_sub[, c("size", "ranking_fulldata")],
+      smmry_vs_trad$perf_sub[, c("size", "ranking_fulldata")],
       info = tstsetup
     )
-    expect_equal(smmry_vs$selection, smmry_vs_trad$selection, tolerance = 1e-6,
+    expect_equal(smmry_vs$perf_sub, smmry_vs_trad$perf_sub, tolerance = 1e-6,
                  info = tstsetup)
   }
 })
@@ -399,23 +403,29 @@ test_that(paste(
     smmry_vs_trad <- smmrys_vs[[tstsetup_trad]]
 
     expect_equal(
-      smmry_vs[setdiff(names(smmry_vs), c("family", "selection"))],
+      smmry_vs[setdiff(names(smmry_vs), c("family", "perf_sub", "perf_ref"))],
       smmry_vs_trad[setdiff(names(smmry_vs_trad),
-                            c("family", "selection"))],
+                            c("family", "perf_sub", "perf_ref"))],
       info = tstsetup
     )
     expect_identical(
-      smmry_vs$selection[, c("size", "solution_terms")],
-      smmry_vs_trad$selection[, c("size", "solution_terms")],
+      smmry_vs$perf_sub[, c("size", "ranking_fulldata", "cv_proportions_diag")],
+      smmry_vs_trad$perf_sub[
+        , c("size", "ranking_fulldata", "cv_proportions_diag")
+      ],
       info = tstsetup
     )
     # Exclude statistics which are not supported for the augmented-data
     # projection:
-    smmry_vs_trad$selection <- smmry_vs_trad$selection[
-      , -grep("mse|auc", names(smmry_vs_trad$selection)), drop = FALSE
+    smmry_vs_trad$perf_sub <- smmry_vs_trad$perf_sub[
+      , -grep("mse|R2|auc", names(smmry_vs_trad$perf_sub)), drop = FALSE
     ]
-    expect_equal(smmry_vs$selection, smmry_vs_trad$selection,
+    smmry_vs_trad$perf_ref <- smmry_vs_trad$perf_ref[
+      -grep("mse|R2|auc", names(smmry_vs_trad$perf_ref))
+    ]
+    expect_equal(smmry_vs$perf_sub, smmry_vs_trad$perf_sub,
                  tolerance = 1e-6, info = tstsetup)
+    expect_equal(smmry_vs$perf_ref, smmry_vs_trad$perf_ref, info = tstsetup)
   }
 })
 
@@ -434,16 +444,6 @@ test_that(paste(
     cvvs_trad <- cvvss[[tstsetup_trad]]
 
     is_kfold <- identical(args_cvvs[[tstsetup]]$cv_method, "kfold")
-    if (is_kfold) {
-      is_glmm <- args_cvvs[[tstsetup]]$mod_nm == "glmm"
-      if (is_glmm) {
-        tol_sub <- 1e-4
-      } else {
-        tol_sub <- 1e-5
-      }
-    } else {
-      tol_sub <- 1e-6
-    }
 
     summs_sub <- cvvs$summaries$sub
     summs_sub_mu <- do.call(cbind, lapply(summs_sub, "[[", "mu"))[
@@ -451,8 +451,18 @@ test_that(paste(
     ]
     summs_sub_trad <- cvvs_trad$summaries$sub
     summs_sub_mu_trad <- do.call(cbind, lapply(summs_sub_trad, "[[", "mu"))
-    expect_equal(summs_sub_mu, summs_sub_mu_trad, tolerance = tol_sub,
-                 info = tstsetup)
+    # Sometimes, there are seemingly large differences on logit scale which are
+    # probably due to numerical overflow of probabilities (i.e., on probability
+    # scale) towards 1 or underflow towards zero. Thus, compare on probability
+    # scale if the comparison on logit scale fails:
+    tryCatch(expect_equal(summs_sub_mu, summs_sub_mu_trad,
+                          tolerance = 1e-6, info = tstsetup),
+             error = function(e) {
+               stopifnot(is_kfold)
+               expect_equal(ilinkfun_raw(summs_sub_mu, link_nm = "logit"),
+                            ilinkfun_raw(summs_sub_mu_trad, link_nm = "logit"),
+                            tolerance = 1e-5, info = tstsetup)
+             })
     summs_sub_lppd <- do.call(cbind, lapply(summs_sub, "[[", "lppd"))
     summs_sub_lppd_trad <- unname(
       do.call(cbind, lapply(summs_sub_trad, "[[", "lppd"))
@@ -462,14 +472,14 @@ test_that(paste(
     # towards 1 or underflow towards zero. Thus, compare on exp scale if the
     # comparison on log scale fails:
     tryCatch(expect_equal(summs_sub_lppd, summs_sub_lppd_trad,
-                          tolerance = tol_sub, info = tstsetup),
+                          tolerance = 1e-6, info = tstsetup),
              error = function(e) {
                stopifnot(is_kfold)
                expect_equal(exp(summs_sub_lppd), exp(summs_sub_lppd_trad),
-                            tolerance = tol_sub, info = tstsetup)
+                            tolerance = 1e-5, info = tstsetup)
              })
     summs_ref <- cvvs$summaries$ref
-    summs_ref$mu <- structure(unclass(summs_ref$mu), nobs_orig = NULL)
+    summs_ref$mu <- structure(unclass(summs_ref$mu), ndiscrete = NULL)
     summs_ref$mu <- summs_ref$mu[(nobsv + 1):(2 * nobsv)]
     expect_equal(summs_ref, cvvs_trad$summaries$ref, tolerance = 1e-12,
                  info = tstsetup)
@@ -493,14 +503,18 @@ test_that(paste(
     smmry_cvvs_trad <- smmrys_cvvs[[tstsetup_trad]]
 
     expect_equal(
-      smmry_cvvs[setdiff(names(smmry_cvvs), c("family", "selection"))],
+      smmry_cvvs[setdiff(names(smmry_cvvs), c("family", "perf_sub"))],
       smmry_cvvs_trad[setdiff(names(smmry_cvvs_trad),
-                              c("family", "selection"))],
+                              c("family", "perf_sub"))],
       info = tstsetup
     )
     expect_identical(
-      smmry_cvvs$selection[, c("size", "solution_terms")],
-      smmry_cvvs_trad$selection[, c("size", "solution_terms")],
+      smmry_cvvs$perf_sub[
+        , c("size", "ranking_fulldata", "cv_proportions_diag")
+      ],
+      smmry_cvvs_trad$perf_sub[
+        , c("size", "ranking_fulldata", "cv_proportions_diag")
+      ],
       info = tstsetup
     )
     is_kfold <- identical(
@@ -508,7 +522,7 @@ test_that(paste(
       "kfold"
     )
     if (is_kfold) {
-      tol_smmry <- 1e-5
+      tol_smmry <- 1e-4
     } else {
       tol_smmry <- 1e-6
     }
@@ -518,27 +532,23 @@ test_that(paste(
       # ELPD:
       stopifnot(is.null(args_smmry_cvvs[[tstsetup]]$stats))
 
-      smmry_pd <- smmry_cvvs$selection
-      smmry_pd[[grep("elpd", names(smmry_pd), value = TRUE)]] <- exp(
-        smmry_pd[[grep("elpd", names(smmry_pd), value = TRUE)]]
-      )
-      smmry_pd$lower <- exp(smmry_pd$lower)
-      smmry_pd$upper <- exp(smmry_pd$upper)
+      smmry_pd <- smmry_cvvs$perf_sub
+      smmry_pd$elpd <- exp(smmry_pd$elpd)
+      smmry_pd$elpd.lower <- exp(smmry_pd$elpd.lower)
+      smmry_pd$elpd.upper <- exp(smmry_pd$elpd.upper)
 
-      smmry_pd_trad <- smmry_cvvs_trad$selection
-      smmry_pd_trad[[grep("elpd", names(smmry_pd_trad), value = TRUE)]] <- exp(
-        smmry_pd_trad[[grep("elpd", names(smmry_pd_trad), value = TRUE)]]
-      )
-      smmry_pd_trad$lower <- exp(smmry_pd_trad$lower)
-      smmry_pd_trad$upper <- exp(smmry_pd_trad$upper)
+      smmry_pd_trad <- smmry_cvvs_trad$perf_sub
+      smmry_pd_trad$elpd <- exp(smmry_pd_trad$elpd)
+      smmry_pd_trad$elpd.lower <- exp(smmry_pd_trad$elpd.lower)
+      smmry_pd_trad$elpd.upper <- exp(smmry_pd_trad$elpd.upper)
 
-      expect_equal(smmry_pd[, setdiff(names(smmry_pd), "se")],
-                   smmry_pd_trad[, setdiff(names(smmry_pd_trad), "se")],
+      expect_equal(smmry_pd[, setdiff(names(smmry_pd), "elpd.se")],
+                   smmry_pd_trad[, setdiff(names(smmry_pd_trad), "elpd.se")],
                    tolerance = tol_smmry, info = tstsetup)
-      expect_equal(smmry_pd$se, smmry_pd_trad$se, tolerance = 1e-1,
+      expect_equal(smmry_pd$elpd.se, smmry_pd_trad$elpd.se, tolerance = 1e-3,
                    info = tstsetup)
     }
-    tryCatch(expect_equal(smmry_cvvs$selection, smmry_cvvs_trad$selection,
+    tryCatch(expect_equal(smmry_cvvs$perf_sub, smmry_cvvs_trad$perf_sub,
                           tolerance = tol_smmry, info = tstsetup),
              error = compare_exp)
   }
@@ -564,61 +574,75 @@ test_that(paste(
     smmry_cvvs_trad <- smmrys_cvvs[[tstsetup_trad]]
 
     expect_equal(
-      smmry_cvvs[setdiff(names(smmry_cvvs), c("family", "selection"))],
+      smmry_cvvs[setdiff(names(smmry_cvvs),
+                         c("family", "perf_sub", "perf_ref"))],
       smmry_cvvs_trad[setdiff(names(smmry_cvvs_trad),
-                              c("family", "selection"))],
+                              c("family", "perf_sub", "perf_ref"))],
       info = tstsetup
     )
     expect_identical(
-      smmry_cvvs$selection[, c("size", "solution_terms")],
-      smmry_cvvs_trad$selection[, c("size", "solution_terms")],
+      smmry_cvvs$perf_sub[
+        , c("size", "ranking_fulldata", "cv_proportions_diag")
+      ],
+      smmry_cvvs_trad$perf_sub[
+        , c("size", "ranking_fulldata", "cv_proportions_diag")
+      ],
       info = tstsetup
     )
     # Exclude statistics which are not supported for the augmented-data
     # projection:
-    smmry_cvvs_trad$selection <- smmry_cvvs_trad$selection[
-      , -grep("mse\\.|auc\\.", names(smmry_cvvs_trad$selection)), drop = FALSE
+    smmry_cvvs_trad$perf_sub <- smmry_cvvs_trad$perf_sub[
+      , -grep("mse|R2|auc", names(smmry_cvvs_trad$perf_sub)), drop = FALSE
+    ]
+    smmry_cvvs_trad$perf_ref <- smmry_cvvs_trad$perf_ref[
+      -grep("mse|R2|auc", names(smmry_cvvs_trad$perf_ref))
     ]
     is_kfold <- identical(
       args_cvvs[[args_smmry_cvvs[[tstsetup]]$tstsetup_vsel]]$cv_method,
       "kfold"
     )
     if (is_kfold) {
-      tol_smmry <- 1e-5
+      tol_smmry <- 1e-4
     } else {
       tol_smmry <- 1e-6
     }
     compare_exp <- function(e) {
       stopifnot(is_kfold)
 
-      smmry_pd <- smmry_cvvs$selection
+      smmry_pd <- smmry_cvvs$perf_sub
       se_cols <- grep("lpd\\.se$", names(smmry_pd), value = TRUE)
       elpd_cols <- setdiff(grep("elpd", names(smmry_pd), value = TRUE),
                            se_cols)
       smmry_pd[elpd_cols] <- exp(smmry_pd[elpd_cols])
       mlpd_cols <- setdiff(grep("mlpd", names(smmry_pd), value = TRUE),
                            se_cols)
+      smmry_pd[mlpd_cols] <- exp(smmry_pd[mlpd_cols])
 
-      smmry_pd_trad <- smmry_cvvs_trad$selection
+      smmry_pd_trad <- smmry_cvvs_trad$perf_sub
       se_cols_trad <- grep("lpd\\.se$", names(smmry_pd_trad), value = TRUE)
-      elpd_cols_trad <- setdiff(grep("elpd", names(smmry_pd_trad), value = TRUE),
-                                se_cols_trad)
+      elpd_cols_trad <- setdiff(
+        grep("elpd", names(smmry_pd_trad), value = TRUE),
+        se_cols_trad
+      )
       smmry_pd_trad[elpd_cols_trad] <- exp(smmry_pd_trad[elpd_cols_trad])
-      mlpd_cols_trad <- setdiff(grep("mlpd", names(smmry_pd_trad), value = TRUE),
-                                se_cols_trad)
+      mlpd_cols_trad <- setdiff(
+        grep("mlpd", names(smmry_pd_trad), value = TRUE),
+        se_cols_trad
+      )
+      smmry_pd_trad[mlpd_cols_trad] <- exp(smmry_pd_trad[mlpd_cols_trad])
 
-      expect_equal(smmry_pd[, setdiff(names(smmry_pd),
-                                      c(mlpd_cols, se_cols))],
-                   smmry_pd_trad[, setdiff(names(smmry_pd_trad),
-                                           c(mlpd_cols_trad, se_cols_trad))],
-                   tolerance = 1e-10, info = tstsetup)
-      expect_equal(smmry_pd[, c(mlpd_cols, se_cols)],
-                   smmry_pd_trad[, c(mlpd_cols_trad, se_cols_trad)],
+      expect_equal(smmry_pd[setdiff(names(smmry_pd), se_cols)],
+                   smmry_pd_trad[setdiff(names(smmry_pd_trad), se_cols_trad)],
+                   tolerance = tol_smmry, info = tstsetup)
+      expect_equal(smmry_pd[se_cols],
+                   smmry_pd_trad[se_cols_trad],
                    tolerance = 1e-3, info = tstsetup)
     }
-    tryCatch(expect_equal(smmry_cvvs$selection, smmry_cvvs_trad$selection,
+    tryCatch(expect_equal(smmry_cvvs$perf_sub, smmry_cvvs_trad$perf_sub,
                           tolerance = tol_smmry, info = tstsetup),
              error = compare_exp)
+    expect_equal(smmry_cvvs$perf_ref, smmry_cvvs_trad$perf_ref,
+                 tolerance = tol_smmry, info = tstsetup)
   }
 })
 
